import pygame
import random
import gameobject
import sys


# This class inherits from player.py and uses
# player's movement functions and game min/maxes
class Computer(gameobject.GameObject):
    def __init__(self,game,starting_position):
        gameobject.GameObject.__init__(self,game,starting_position) # superclass constructor

        # These parameters will be unique to the computer
        self.xmove = 3 #movement rate per tick
        self.ymove = 3

        self.move_same_as_before = self.move_up

    def _nospam(self,text):
        #sys.stdout.write('\r' + text + ' ')
        #sys.stdout.flush()
        pass

    def move(self):
        # this function is run once every game loop to move the computer
        self.halted = self.game.computer.colliding
        self.prioritize()
        
    def prioritize(self):
        my_x = self.game.computer.y
        my_y = self.game.computer.x
        p_x = self.game.player.x
        p_y = self.game.player.y
        # 1. determine what constitutes being "close" to player
        # 2. order of logic:
        #   determine which of gather,attack,upgrade,runaway is best
        #   if you have enough powerups to upgrade and player is not close, upgrade
        #   elif you are significantly more powerful than player, then attack
        #   elif you have enough powerups to upgrade and player is close, runaway
        #   else gather unless player is close; otherwise run away


        # we have powerups stuck in walls - really  messes up AI right now 
        if self.game.player.powerups == 0:
            self._nospam('[Computer] Player is a big zero! [ %d %d ]  colliding?: %s | computer status: %s | player status: %s' % (my_x, my_y, self.halted,self.game.computer.powerups,self.game.player.powerups))
            self.move_none()

        elif self.game.player.powerups > self.game.computer.powerups and self.game.computer.powerups >= 3:

            # replace this with "if player is close to me, run away, otherwise go get stuff to get stronger"
            if my_x or my_y <= p_x + 5  or p_y + 5:
                self._nospam('[Computer] Player is scary! [ %d %d ]  colliding?: %s | computer status: %s | player status: %s' % (my_x, my_y, self.halted,self.game.computer.powerups,self.game.player.powerups))
                self.runaway()
            else:
                self._nospam('[Computer] (not scared)  Getting closest powerup [ %d %d ]  colliding?: %s | computer status: %s | player status: %s' % (my_x, my_y, self.halted,self.game.computer.powerups,self.game.player.powerups))
                self.gather__find_closest_powerup()


        elif self.game.computer.powerups >= 5 and self.game.computer.powerups > self.game.player.powerups:
            self._nospam('[Computer] Attacking player! [ %d %d ]  colliding?: %s | computer status: %s | player status: %s' % (my_x, my_y, self.halted,self.game.computer.powerups,self.game.player.powerups))
            self.attack_player() 

        elif self.game.computer.powerups == self.game.player.powerups:
            self._nospam('[Computer] Getting closest powerup [ %d %d ]  colliding?: %s | computer status: %s | player status: %s' % (my_x, my_y, self.halted,self.game.computer.powerups,self.game.player.powerups))
            self.gather__find_closest_powerup()
        else:
            self._nospam('[Computer] Getting closest powerup [ %d %d ]  colliding?: %s | computer status: %s | player status: %s' % (my_x, my_y, self.halted,self.game.computer.powerups,self.game.player.powerups))
            self.gather__find_closest_powerup()


        #self._nospam('\r' + str(self.game.computer.__dict__))
        #i = random.randint(1,9)
        #if i%2==0:
        #    self.attack__find_player_shortest_path()
        #else:
        #    self.move_random()

    def panic(self):
        if self.halted:
            for i in range(random.randint(300,500)):
                self.move_up()
                self.move_left()
            for i in range(random.randint(300,500)):
                self.move_down()
                self.move_right()
        
    def move_random(self):
        i = random.randint(1,9)
        if i==1:
            self.move_up()
            self.move_same_as_before = self.move_up
        elif i==2:
            self.move_down()
            self.move_same_as_before = self.move_down
        elif i==3:
            self.move_left()
            self.move_same_as_before = self.move_left
        elif i==4:
            self.move_right()
            self.move_same_as_before = self.move_right
        elif i==5 or i==6 or i==7:
            self.move_none()
            self.move_same_as_before = self.move_none
        else:
            self.move_same_as_before
    
    def move_none(self):
        pass

    def gather(self):
        pass
       
    # renamed this because the method collides with the attack variable in gameobject    
    def attack_player(self):
        self.attack__find_player_shortest_path()
        
    def upgrade(self):
        pass
        
    def runaway(self):
        self.runaway__find_player_shortest_path()
        
    def gather__find_closest_powerup(self):
        closest_powerup = self.game.map.width*self.game.map.height #start with a really far distance away
        x_dir = 0
        y_dir = 0
        for i in self.game.gameobjects:
            # ignore computer, player and powerups that intersect walls or other powerups.
            if i==self.game.player or i==self.game.computer or i.colliding: continue
            temp_x,temp_y = self.game.map.find_fastest_direct_path((self.cx,self.cy), (i.cx,i.cy))
            temp_dist = self.game.map.find_distance(0,0,temp_x,temp_y)
            if temp_dist<closest_powerup:
                closest_powerup=temp_dist
                x_dir = temp_x
                y_dir = temp_y
        if x_dir > 0:
            if self.halted:
                for i in range(100): self.move_left()
            else:
                self.move_right()
        elif x_dir<0:
            if self.halted:
                for i in range(100): self.move_right()
            else:
                self.move_left()
        if y_dir>0:
            if self.halted:
                for i in range(100): self.move_up()
            else:
                self.move_down()
        elif y_dir<0:
            if self.halted:
                for i in range(100): self.move_down()
            else:
                self.move_up()

       

    def runaway__find_player_shortest_path(self):
        x_dir,y_dir = self.game.map.find_fastest_direct_path((self.x,self.y), \
                                        (self.game.player.x,self.game.player.y))
        if x_dir > 0:
            self.move_left()
        elif x_dir<0:
            self.move_right()
        if y_dir>0:
            self.move_up()
        elif y_dir<0:
            self.move_down()

            
    def attack__find_player_shortest_path(self):
        x_dir,y_dir = self.game.map.find_fastest_direct_path((self.x,self.y), \
                                        (self.game.player.x,self.game.player.y))
        if x_dir > 0:
            self.move_right()
        elif x_dir<0:
            self.move_left()
        if y_dir>0:
            self.move_down()
        elif y_dir<0:
            self.move_up()

