import os
import random

import map
import data
import view
import pygame
import player
import powerup
import computer
import mutation
import minimapobject

'''
Game is a model of the game data that can be controlled
by the included controllers

'''
class Game:
    def __init__(self):
        # All game objects are inside this class

        # load view before game because display is set here
        self.view = view.View(self,800,600)

        # Load Game Graphics, Maps and other Resources
        self.resource = dict()
        self._load_resources()
        self.death = False
        self.win = False
        self.mutation_in_progress = False
        self.mutation = None
        self.player = None
        self.computer = None
        self.computer2 = None
        self.gameobjects = [] # all game objects to be drawn on map
        self.show_instructions = True
        
        # Level 0 is the menu level
        # set this for testing purposes
        # first parameter is level, second is player's choice of icon
        self.load_level(0,'white_circle')
        #self.laod_level(4,'SpeedBot')

    def _load_resources(self):
        self.resource['menu_bg'] = self._load_image("menu_bg.jpg")
        self.resource['rain_bg'] = self._load_image("rain_bg.jpg",(800,600))
        self.resource['water1'] = self._load_image("water1.jpg",(800,600))
        self.resource['fg1'] = self._load_image("fg1.jpg")
        #self.resource['bg-stars'] = self._load_image("bg-stars.jpg")
        #self.resource['bg1'] = self._load_image("bg1.jpg")
        #self.resource['bg2'] = self._load_image("bg2.jpg")
        self.resource['bg4'] = self._load_image("space.jpg",(800,600))
        #self.resource['stars1'] = self._load_image("stars1.png",alpha=50)
        self.resource['Forest-1'] = self._load_image("Forest-1.jpg")
        self.resource['Forest-2'] = self._load_image("Forest-2.jpg")
        self.resource['ice'] = self._load_image("ice.jpg")
        self.resource['lava'] = self._load_image("lava.jpg")
        self.resource['asteroid'] = self._load_image("asteroid.png")

        # load sprites
        self.resource['test_player'] = self._load_image("test_player.jpg",(50,100))
        self.resource['p_white_circle'] = self._load_image("white_circle.png",(50,50))

        # load powerups
        # speed
        self.resource['speed_powerup_normal'] = self._load_image("green_circle.png", (25,25))
        self.resource['speed_powerup_up'] = self._load_image("green_circle.png", (25,30))
        self.resource['speed_powerup_down'] = self._load_image("green_circle.png", (25,20))
        # attack
        self.resource['attack_powerup_normal'] = self._load_image("red_circle.png", (25,25))
        self.resource['attack_powerup_up'] = self._load_image("red_circle.png", (25,30))
        self.resource['attack_powerup_down'] = self._load_image("red_circle.png", (25,20))
        # defense
        self.resource['defense_powerup_normal'] = self._load_image("blue_circle.png", (25,25))
        self.resource['defense_powerup_up'] = self._load_image("blue_circle.png", (25,30))
        self.resource['defense_powerup_down'] = self._load_image("blue_circle.png", (25,20))

        # level 1 mutations
        self.resource['ch_Squirrel'] = self._load_image("ch_squirrel.png",(200,450),calpha=False) #speed
        self.resource['ch_Badger'] = self._load_image("ch_badger.png",(200,450),calpha=False) #attack
        self.resource['ch_Porcu'] = self._load_image("ch_porcupine.png",(200,450),calpha=False) #defense
        self.resource['p_Squirrel'] = self._load_image("p_squirrel.png") #speed
        self.resource['p_Badger'] = self._load_image("p_badger.png") #attack
        self.resource['p_Porcu'] = self._load_image("p_porcupine.png") #defense
        
        # level 2 mutations
        self.resource['ch_Lion'] = self._load_image("ch_lion.png",(200,450),calpha=False) #defense
        self.resource['ch_Rhino'] = self._load_image("ch_rhino.png",(200,450),calpha=False) #defense
        self.resource['ch_Cheetah'] = self._load_image("ch_cheetah.png",(200,450),calpha=False) #defense
        self.resource['p_Lion'] = self._load_image("p_lion.png") #defense
        self.resource['p_Rhino'] = self._load_image("p_rhino.png") #defense
        self.resource['p_Cheetah'] = self._load_image("p_cheetah.png") #defense

        # level 3 mutations
        self.resource['ch_AttackBot'] = self._load_image("ch_attack_bot.png",(200,450),calpha=False) #defense
        self.resource['ch_DefenseBot'] = self._load_image("ch_defense_bot.png",(200,450),calpha=False) #defense
        self.resource['ch_SpeedBot'] = self._load_image("ch_speed_bot.png",(200,450),calpha=False) #defense
        self.resource['p_AttackBot'] = self._load_image("p_attack_bot.png") #defense
        self.resource['p_DefenseBot'] = self._load_image("p_defense_bot.png") #defense
        self.resource['p_SpeedBot'] = self._load_image("p_speed_bot.png") #defense


    def _load_image(self,filename,size=None,alpha=255,calpha=True):
        if size==None:
            if calpha:
                retval = pygame.image.load(data.filepath(filename)).convert_alpha()
            else:
                retval = pygame.image.load(data.filepath(filename)).convert()
        else:
            if calpha:
                retval = pygame.transform.scale(pygame.image.load(data.filepath(filename)),size).convert_alpha()
            else:
                retval = pygame.transform.scale(pygame.image.load(data.filepath(filename)),size).convert()
        retval.set_alpha(alpha)
        return retval


    def load_level(self,level,choice):
        #print "Loading Level:"+str(level)
        # reset level
        self.choice = choice
        self.show_instructions = True
        self.view.instr_alpha = 255
        self.level = level
        self.gameobjects = []
        self.powerups = 0
        #self.player = None
        #self.computer = None
        self.mutation = None
        self.mutation_in_progress = False
        self.death = False
        self.win = False
        self.map = map.Map('map'+str(level)+'.csv',self,level)

        # game object array has all in-game objects in it
        if level==0: #main menu
            menu_obj = powerup.Powerup(self,(300,self.view.cy))
            self.gameobjects.append(menu_obj)

        elif level==1: #primordial soup
            self.powerups = 40
            self.player = player.Player(self,(170,170))
            self.player.max_powerups = self.powerups
            self.player.image = self.resource["p_"+choice]
            self.player.speed = 3
            self.mutation = mutation.Mutation(self,('Badger','Porcu','Squirrel'))
            self.gameobjects.append(self.player)
            self.add_random_powerups(self.powerups)
            for i in self.gameobjects:
                if i==self.player: continue
                i.vx = 1
        
        elif level==2: #forest
            self.powerups = 40
            self.player = player.Player(self,(170,170))
            self.player.image = self.resource["p_"+choice]
            if self.choice=='Badger': self.player.attack += 5
            elif self.choice=='Porcu': self.player.defense += 5
            elif self.choice=='Squirrel': self.player.speed += 5
            self.computer = computer.Computer(self,(300,300))
            self.mutation = mutation.Mutation(self,('Lion','Rhino','Cheetah'))
            im = random.randint(0,2)
            if im==0:self.computer.image = self.resource['p_Badger'] 
            elif im==1:self.computer.image = self.resource['p_Porcu']
            else: self.computer.image = self.resource['p_Squirrel']
            self.gameobjects.append(self.player)
            self.gameobjects.append(self.computer)
            self.add_random_powerups(self.powerups)
        
        elif level==3: #mountain/lava
            self.powerups = 30
            if not self.player: self.player = player.Player(self,(0,0))
            if not self.computer: self.computer = computer.Computer(self,(0,0))
            self.player.x = 170
            self.player.y = 170
            self.player.image = self.resource["p_"+choice]
            self.player.health += 1000
            if self.choice=='Lion': self.player.attack += 10
            elif self.choice=='Rhino': self.player.defense += 10
            elif self.choice=='Cheetah': self.player.speed += 10
            self.computer.x = 1500
            self.computer.y = 300
            self.computer.health += 1000
            self.mutation = mutation.Mutation(self,('AttackBot','DefenseBot','SpeedBot'))
            im = random.randint(0,2)
            if im==0:self.computer.image = self.resource['p_Lion'] 
            elif im==1:self.computer.image = self.resource['p_Rhino']
            else: self.computer.image = self.resource['p_Cheetah']
            self.gameobjects.append(self.player)
            self.gameobjects.append(self.computer)
            self.add_random_powerups(self.powerups)

        elif level==4: #future/robots/lasers
            self.powerups = 4
            if not self.player: self.player = player.Player(self,(0,0))
            if not self.computer: self.computer = computer.Computer(self,(0,0))
            self.player.x = 170
            self.player.y = 170
            self.player.image = self.resource["p_"+choice]
            self.player.health += 2000
            if self.choice=='AttackBot': self.player.attack += 20
            elif self.choice=='DefenseBot': self.player.defense += 20
            elif self.choice=='SpeedBot': self.player.speed += 20
            self.computer.x = 1300
            self.computer.y = 300
            self.computer.health += 2000
            self.computer2 = computer.Computer(self,(2000,100))
            self.computer.health += 400
            self.mutation = None
            im = random.randint(0,2)
            if im==0: self.computer.image = self.resource['p_AttackBot'] 
            elif im==1: self.computer.image = self.resource['p_DefenseBot']
            else: self.computer.image = self.resource['p_SpeedBot']
            im2 = random.randint(0,2)
            if im2==0: self.computer2.image = self.resource['p_AttackBot'] 
            elif im2==1: self.computer2.image = self.resource['p_DefenseBot']
            else: self.computer2.image = self.resource['p_SpeedBot']
            self.gameobjects.append(self.player)
            self.gameobjects.append(self.computer)
            self.gameobjects.append(self.computer2)
            self.add_random_powerups(self.powerups)

        else: #level 5+ means you win, insert animation here
            self.win = True
        
        # minimap and associated sprites
        self.minimap_x = 15
        self.minimap_y = 15
        self.minimap_scale = 0.05 # 5% of actual size
        # larger map = larger minimap and should preserve dimensions
        # caution: very large row/column count ( 200+ or so ) will make the minimap huge.
        self.minimap = minimapobject.MinimapObject((0,0,0), \
            (self.minimap_x,self.minimap_y), \
            (self.map.width * self.minimap_scale ,self.map.height * self.minimap_scale))
        self.player_mini = minimapobject.MinimapObject((100,100,255),self.minimap.rect.center,(5,5))
        self.computer_mini = minimapobject.MinimapObject((255,0,0),self.minimap.rect.center,(5,5))

        # statscreen
        self.statscreen = minimapobject.MinimapObject((0,0,0),(15,450),(200,135))


    def mutation_choice(self,choice):
        self.choice = choice
        self.level_up()
        self.mutation_in_progress = False


    def add_random_powerups(self,num):
        for i in range(num): # place x random powerups
            while 1: # careful, could cause hang up 
                pos = random.randint(0,self.map.width),random.randint(0,self.map.height)
                pup = powerup.Powerup(self,pos)
                pos_rect = pup.getrect()
                pos_rect.inflate(4,4) # keep powerups away from walls
                if not self.map.collision(pos_rect) \
                        and (not self.detect_collision(self.player.getrect(),pos_rect) if self.player else 1) \
                        and (not self.detect_collision(self.computer.getrect(),pos_rect) if self.computer else 1):
                    self.gameobjects.append(powerup.Powerup(self,pos))
                    break


    def tick(self,input):
        # called every game tick

        # if mutating, send tick to mutation object
        if self.mutation_in_progress:
            if self.mutation:
                self.mutation.tick(input)
                self.view.update(input)
                return

        # define level completion goal here
        if self.level==0: #main menu
            if input[0]=='e' or input[1][0]: 
                self.level_up()
        elif self.level==1:
            if self.player.powerups >= self.player.max_powerups:
                self.level_up()
        elif self.level==2:
            if self.player.health<1:
                self.death=True
            elif self.computer.health<1:
                self.level_up()
            elif self.computer.powerups>20 or self.player.powerups>20:
                self.level_up()
        elif self.level==3:
            if self.player.health<1:
                self.death=True
            elif self.computer.health<1:
                self.level_up()
        elif self.level==4:
            if self.player.health<1:
                self.death=True
            elif self.computer.health<1:
                self.level_up()

        if self.death:
            #run death animation
            self.player.health = 100
            self.load_level(self.level,self.choice)

        elif self.win:
            #run win animation 
            print "you win!!!!!!!"
            pass

        for i in self.gameobjects: # call move function for all objects
            if i==self.player:
                i.move(input,self.level)
            else:
                i.move()

        # Detect object collisions with player and handle them
        self.detect_player_collisions()

        # Update the visible screen
        self.view.update(input)


    def game_over(self):
        pass


    def level_up(self):
        if not self.mutation_in_progress and self.level>0:
            self.mutation_in_progress = True
        else:
            self.level = self.level + 1
            if self.level==5:
                self.win = True
            else:
                self.load_level(self.level,self.choice)


    def detect_player_collisions(self):
        # handle attacking between player and computer here
        if self.player and self.computer:
            if self.detect_collision(self.player.getrect(),self.computer.getrect()):
                if self.computer.attack-self.player.defense/2 > 0:
                    self.player.health -= (self.computer.attack - self.player.defense/2)
                if self.player.attack-self.computer.defense/2 > 0:
                    self.computer.health -= (self.player.attack - self.computer.defense/2)

        if self.player and self.computer2:
            if self.detect_collision(self.player.getrect(),self.computer2.getrect()):
                if self.computer2.attack-self.player.defense/2 > 0:
                    self.player.health -= (self.computer2.attack - self.player.defense/2)
                if self.player.attack-self.computer2.defense/2 > 0:
                    self.computer2.health -= (self.player.attack - self.computer2.defense/2)

        # handle player or computer collecting objects here
        for i in self.gameobjects:
            if i==self.player or i==self.computer or i==self.computer2:
                continue #not interested in comparing to self
                
            if self.player:    
                if self.detect_collision(self.player.getrect(),i.getrect()):
                    # player collected a powerup
                    # Remove powerup from list
                    # add stats to player
                    # set time period of no activity for player
                    self.player.powerup_collected(i.stat)
                    self.gameobjects.remove(i)
                    continue # prevents same object from being taken by computer and player at same time

            if self.computer:
                if self.detect_collision(self.computer.getrect(),i.getrect()):# elif: touching tie goes to the player
                    # computer collected a powerup
                    # Remove powerup from list
                    # add stats to computer
                    # set time period of no activity for computer
                    self.computer.powerup_collected(i.stat)
                    self.gameobjects.remove(i)
                    continue

            if self.computer2:
                if self.detect_collision(self.computer2.getrect(),i.getrect()):
                    # computer collected a powerup
                    # Remove powerup from list
                    # add stats to computer
                    # set time period of no activity for computer
                    self.computer.powerup_collected(i.stat)
                    self.gameobjects.remove(i)
                    continue

    def detect_collision(self,rect1,rect2):
        # Determine if there has been a collision between passed-in rect and any walls
        if rect1.colliderect(rect2):
            return True
        offsety = -self.map.height if rect1.bottom > self.map.height else 0
        offsetx = -self.map.width if rect1.right > self.map.width else 0
        rect1.move_ip(offsetx,0)
        if rect1.colliderect(rect2):
            return True
        rect1.move_ip(0,offsety)
        if rect1.colliderect(rect2):
            return True
        rect1.move_ip(-offsetx,0)
        if rect1.colliderect(rect2):
            return True            
        return False
