import pygame

class GameObject:
    def __init__(self,game,starting_position):
        self.game = game
        self.xmin = 0 #furthest upper left corner
        self.ymin = 0
        self.xmax = self.game.map.width #furthest lower right corner, minus display size
        self.ymax = self.game.map.height
        self.x,self.y = starting_position

        self.xmove = 2.0 #movement rate per tick
        self.ymove = 2.0

        # image load
        self.image = self.game.resource['test_player']

        self.cx = self.x + self.width()/2 # center point of object
        self.cy = self.y + self.height()/2

        self.vx = 0.0 # velocity
        self.vy = 0.0
        self.ax = 0.0 # acceleration
        self.ay = 0.0
        
        # These represent how many powerups were collected or converted to attributes
        self.health = 200
        self.attack = 0
        self.defense = 0
        self.speed = 0
        self.powerups = 0


        # values for calculations
        self.speed_when_exhausted = 1
        self.speed_bonus = 0
        self.attack_bonus = 0
        self.defense_bonus = 0
        self.max_powerups = 0

        # set colliding
        self.colliding = False

    def move(self):
        # this function is called whenever the game ticks
        self.apply_physics()

    def apply_physics(self):
        self.vx += self.ax
        self.vy += self.ay
        self.move_xy(self.vx,self.vy) # add vx to x, vy to y
        
    def move_xy(self,mx,my):
        # generic move command, assumes mx and my form the angle of movement
        # we still need to figure out speed and collisions here
        self.y = int((self.y + my) % self.ymax)
        if self.detect_wall_collision():
            self.y = int((self.y - my) % self.ymax)
            self.vy = 0
            self.ay = 0
        self.x = int((self.x + mx) % self.xmax)
        if self.detect_wall_collision():
            self.x = int((self.x - mx) % self.xmax)
            self.vx = 0
            self.ax = 0
        self.player_moved()
    
    
    def move_toward(self,mx,my):
        # move generally in the direction defined by the vector (mx,my) in one movement iteration
        _mx = 2
        _my = (my/mx)*2
        self.move_xy(mx,my)


    def move_up(self,mv=None):
        if mv==None: mv=self.ymove+(self.speed/8)
        for i in range(mv):
            self.move_xy(0,-1)

    def move_down(self,mv=None):
        if mv==None: mv=self.ymove+(self.speed/8)
        for i in range(mv):
            self.move_xy(0,1)
            
    def move_left(self,mv=None):
        if mv==None: mv=self.xmove+(self.speed/8)
        for i in range(mv):
            self.move_xy(-1,0)

    def move_right(self,mv=None):
        if mv==None: mv=self.xmove+(self.speed/8)
        for i in range(mv):
            self.move_xy(1,0)

    def player_moved(self):
        # this function is run whenever player moves to account for center of object
        self.cx = (self.x + self.width()/2) % self.game.map.width # center point of object
        self.cy = (self.y + self.height()/2) % self.game.map.width

    def width(self):
        return self.image.get_width()

    def height(self):
        return self.image.get_height()

    def getrect(self):
        return pygame.Rect(self.x,self.y,self.width(),self.height())
        
    def detect_wall_collision(self):
        rect = self.getrect()
        if self.game.map.collision(rect):
            self.colliding = True
            return True
        else:
            self.colliding = False
        return False

    def powerup_collected(self,stat):
        self.powerups = self.powerups + 1
        if stat == "speed":
            self.increase_speed()
        elif stat == "attack":
            self.increase_attack()
        elif stat == "defense":
            self.increase_defense()

    def increase_speed(self):
        if self.speed_bonus > 13:
            self.speed_bonus = self.speed_bonus + 1 
        elif self.speed_bonus==12:
            self.speed_bonus = self.speed_bonus + 2
        elif self.speed_bonus==8:
            self.speed_bonus = self.speed_bonus + 4
        else:
            self.speed_bonus = self.speed_bonus + 8
        self.speed = self.speed_bonus

    def increase_attack(self):
        if self.attack_bonus > 13:
            self.attack_bonus = self.attack_bonus + 1 
        elif self.attack_bonus==12:
            self.attack_bonus = self.attack_bonus + 2
        elif self.attack_bonus==8:
            self.attack_bonus = self.attack_bonus + 4
        else:
            self.attack_bonus = self.attack_bonus + 8
        self.attack = self.attack_bonus
        
    def increase_defense(self):
        if self.defense_bonus > 13:
            self.defense_bonus = self.defense_bonus + 1 
        elif self.defense_bonus==12:
            self.defense_bonus = self.defense_bonus + 2
        elif self.defense_bonus==8:
            self.defense_bonus = self.defense_bonus + 4
        else:
            self.defense_bonus = self.defense_bonus + 8
        self.defense = self.defense_bonus
