import pygame
import keyboard
import powerup

class View:
    
    def __init__(self,game,width,height):
        self.game=game
        self.width=width
        self.height=height
        self.cx = self.width/2-1
        self.cy = self.height/2-1

        # this is the magic that sets up the display window
        self.screen = pygame.display.set_mode((self.width,self.height), \
            pygame.DOUBLEBUF | pygame.HWSURFACE ) #| pygame.NOFRAME 

        # make stat font
        self.statfont = pygame.font.SysFont("None",25)

        # make title page view objects
        self.title_font = pygame.font.Font(None,160)
        self.subtext_font = pygame.font.Font(None,32)

        # Setup text screens for menu and levels
        self.pyweek_text = self.subtext_font.render( \
            "A game created for pyweek 13 (September 10, 2011)",1,(255,255,0))
        self.option_1_text = self.subtext_font.render( \
            "Start Game",1,(255,255,255))
        self.instructions_text = self.subtext_font.render( \
            "Press Enter Key to continue, or Esc to quit",1,(0,0,100))
        self.acw_mutate_text = self.title_font.render( \
            "ACW Mutate!",1,(0,0,0))
        self.level1_text = self.subtext_font.render( \
            "Level 1: Use the arrow keys.  Collect the powerups to advance.",1,(0,0,0),(255,255,255))
        self.level2_text = self.subtext_font.render( \
            "Level 2: Collect 20 powerups before the computer does!",1,(0,0,0),(255,255,255))
        self.level3_text = self.subtext_font.render( \
            "Level 3: Collect powerups and attack the computer!",1,(0,0,0),(255,255,255))
        self.instr_alpha = 255
        self.win_text = self.title_font.render("You Won!",1,(0,0,0))
        self.win_text2 = self.subtext_font.render("Press Esc to quit",1,(0,0,100))
   

    def update(self,input):

        # determine the view coordinates based on the player coordinates
        viewx = self.game.player.cx-self.cx if self.game.player else 0
        viewy = self.game.player.cy-self.cy if self.game.player else 0
        if viewx<-self.cx: viewx += self.game.map.width
        if viewy<-self.cy: viewy += self.game.map.height
        if viewx>self.game.map.width-self.width:  #removing for testing
            viewx = viewx - self.game.map.width
        if viewy>self.game.map.height-self.height: 
            viewy = viewy - self.game.map.height
        self.screen.blit(self.game.map.get_viewable(viewx,viewy, \
                                                    self.width,self.height),(0,0))
        
        # main menu only
        if self.game.level==0:
            self.screen.blit(self.acw_mutate_text,(self.cx-self.acw_mutate_text.get_width()/2,150))
            self.screen.blit(self.pyweek_text,(self.cx-self.pyweek_text.get_width()/2,400))
            self.screen.blit(self.option_1_text,(self.cx-self.option_1_text.get_width()/2,self.cy))
            self.screen.blit(self.instructions_text,(self.cx-self.instructions_text.get_width()/2,550))

        # draw all game objects on screen
        for i in self.game.gameobjects:
            objx = (i.x-viewx+i.width()) % self.game.map.width-i.width()
            objy = (i.y-viewy+i.height()) % self.game.map.height-i.height()
            '''
            if objx<0: objx += self.game.map.width
            if objy<0: objy += self.game.map.height
            if objx>self.game.map.width-self.width: 
                objx -= self.game.map.width
            if objy>self.game.map.height-self.height: 
                objy -= self.game.map.height
            if i==self.game.player:
                print objx,objy
            '''
            self.screen.blit(i.image,(objx,objy))
        
        # Draw HUD

        # Draw minimap
        # player minimap
        playerdot = self.game.player_mini
        playerdot.image.set_alpha(150)

        # computer minimap
        computerdot = self.game.computer_mini
        computerdot.image.set_alpha(150)
    
        # minimap itself
        minimap = self.game.minimap
        minimap.image.set_alpha(150)
        minimap.rect.move(minimap.position)

        # draw minimap, player, computer in that order.  we don't really keep track of layers anywhere, so order matters.
        # whatever you draw _last_ is what ends up on top
        if self.game.player:
            self.screen.blit(minimap.image,minimap.position)
            dot_p_x = self.game.player.x * self.game.minimap_scale + minimap.position[0]
            dot_p_y = self.game.player.y * self.game.minimap_scale + minimap.position[1]
            self.screen.blit(playerdot.image,(dot_p_x,dot_p_y))
        
        if self.game.computer:
            dot_c_x = self.game.computer.x * self.game.minimap_scale + minimap.position[0]
            dot_c_y = self.game.computer.y * self.game.minimap_scale + minimap.position[1]
            self.screen.blit(computerdot.image,(dot_c_x,dot_c_y))

        
        # draw statscreen, health, stamina, attack, defense, and in that order.
        if self.game.level>0:
            statscreen = self.game.statscreen
            statscreen.image.set_alpha(150)
            self.screen.blit(statscreen.image,statscreen.position)
            if self.game.level>1:
                health = self.game.player.health
                attack = self.game.player.attack
                defense = self.game.player.defense
                speed = self.game.player.speed
                self.screen.blit(self.statfont.render(("Health: " + str(health)),0, \
                    (255,255,255)),(30,465))
                self.screen.blit(self.statfont.render(("Attack: " + str(attack)),0, \
                    (255,0,0)),(30,515))
                self.screen.blit(self.statfont.render(("Defense: " + str(defense)),0, \
                    (0,0,255)),(30,540))
                self.screen.blit(self.statfont.render(("Speed: " + str(speed)),0, \
                    (0,255,0)),(30,565))
            else:
                powerups = self.game.player.powerups
                powerups_max = self.game.player.max_powerups
                self.screen.blit(self.statfont.render(("Powerups: " + str(powerups)) + \
                    "/"+str(powerups_max),0,(255,255,255)),(30,465))

            if self.game.show_instructions:
                if self.game.level==1:
                    self.level1_text.set_alpha(self.instr_alpha)
                    self.screen.blit(self.level1_text, \
                        (self.cx-self.level1_text.get_width()/2,self.cy-45))
                elif self.game.level==2:
                    self.level2_text.set_alpha(self.instr_alpha)
                    self.screen.blit(self.level2_text, \
                        (self.cx-self.level2_text.get_width()/2,self.cy-45))
                elif self.game.level==3:
                    self.level3_text.set_alpha(self.instr_alpha)
                    self.screen.blit(self.level3_text, \
                        (self.cx-self.level3_text.get_width()/2,self.cy-45))
                
                self.instr_alpha -= 2
                if self.instr_alpha < 0 or self.game.mutation_in_progress:
                    self.instr_alpha = 0
                    self.game.show_instructions = False

        # determine whether a mutation display is necessary
        if self.game.mutation_in_progress:
            if self.game.mutation:
                self.game.mutation.update()
       
        # determine if we've won
        if self.game.win==True:
            self.screen.fill((255,255,255))
            self.screen.blit(self.win_text,(self.cx-self.win_text.get_width()/2,150))
            self.screen.blit(self.win_text2,(self.cx-self.instructions_text.get_width()/2,550))
        
        pygame.display.flip()


